from django.db import models
from account.models import Account
from products.models import Product
from suppliers.models import Supplier

class PurchaseOrder(models.Model):
    order_id = models.AutoField(primary_key=True)
    supplier = models.ForeignKey(Supplier, on_delete=models.SET_NULL, null=True, related_name='purchase_orders')
    order_date = models.DateTimeField(auto_now_add=True)
    delivery_date = models.DateTimeField(null=True, blank=True)

    STATUS_CHOICES = [
        ('Pending', 'Pending'),
        ('Approved', 'Approved'),
        ('Shipped', 'Shipped'),
        ('Delivered', 'Delivered'),
        ('Cancelled', 'Cancelled'),
        ('Received', 'Received'),
        ('Partially Received', 'Partially Received'),
    ]
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='Pending')
    total_cost = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def calculate_total_cost(self):
        purchases = self.purchases.all()
        total = sum(purchase.total_cost for purchase in purchases)
        self.total_cost = total
        self.save()

    def __str__(self):
        return f'Order {self.order_id} - {self.supplier.supplier_name}'

    class Meta:
        db_table = "tbl_purchase_order"


class Purchase(models.Model):
    purchase_order = models.ForeignKey(PurchaseOrder, on_delete=models.CASCADE, related_name='purchases', null=True, blank=True)
    product = models.ForeignKey(Product, on_delete=models.CASCADE, related_name='purchases')
    quantity = models.PositiveIntegerField()
    Q_received = models.IntegerField(default=0)
    price_per_unit = models.DecimalField(max_digits=10, decimal_places=2)
    total_cost = models.DecimalField(max_digits=10, decimal_places=2, editable=False)

    STATUS_CHOICES = [
        ('Pending', 'Pending'),
        ('Completed', 'Completed'),
        ('Cancelled', 'Cancelled'),
    ]
    status = models.CharField(max_length=10, choices=STATUS_CHOICES, default='Pending')

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def save(self, *args, **kwargs):
        self.total_cost = self.quantity * self.price_per_unit
        super(Purchase, self).save(*args, **kwargs)

        if self.purchase_order:
            self.purchase_order.calculate_total_cost()

    def __str__(self):
        return f'Purchase {self.status} - {self.product.product_name}'
